
--**-------*** Exerceo ***-------**--
--** Created by Excudo (2014-05) **--
--**-----------------------------**--

import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Excudo.Exerceo.Class";

-- Player --
MyPlayer = class(Turbine.Gameplay.LocalPlayer.GetInstance());

	function MyPlayer:Constructor()

		self:Reset();
		
	end
	
	function MyPlayer:GetResults()
	
		outString = "*** " .. self:GetName() .. " ***\n";
		outString = outString .. "\nState: " .. self.m_state .. "\n";
		outString = outString .. "\nIn Combat (s): " .. tostring(self.m_inCombatSeconds);
		outString = outString .. "\n\nTotal Damage: " .. tostring(self.m_totalDamage);
		outString = outString .. "\nDPS: " .. tostring(string.format("%.2f", self.m_dps));
		outString = outString .. "\nBest Hit: " .. tostring(self.m_bestHit);
		outString = outString .. "\nTotal Damage Taken: " .. tostring(self.m_totalDamageTaken);
		outString = outString .. "\n\nTotal Healing: " .. tostring(self.m_totalHealing);
		outString = outString .. "\nBest Heal: " .. tostring(self.m_bestHeal);
		
		return outString;
		
	end
	
	function MyPlayer:Reset()
	
		-- Attributes --
		self.m_state = "Off";
		self.m_inCombatSeconds = 0;
		self.m_dps = 0;
		self.m_totalDamage = 0;
		self.m_bestHit = 0;
		self.m_totalDamageTaken = 0;
		self.m_totalHealing = 0;
		self.m_bestHeal = 0;
		
	end

	function MyPlayer:UpdateDPS()
	
		self.m_dps = self.m_totalDamage / self.m_inCombatSeconds;
	
	end
	
	function MyPlayer:UpdateDamage(damage, taken)
		
		if taken == false then -- I did it...
		
			self.m_totalDamage = self.m_totalDamage + damage;
			
			self:UpdateDPS();
			
			if damage > self.m_bestHit then
			
				self.m_bestHit = damage;
			
			end
			
		else -- The mob did it...
		
			self.m_totalDamageTaken = self.m_totalDamageTaken + damage;
		
		end
		
	end
	
	function MyPlayer:UpdateHealing(heal)
		
		self.m_totalHealing = self.m_totalHealing + heal;
			
		if heal > self.m_bestHeal then
			
			self.m_bestHeal = heal;
			
		end
		
	end	
	
-- Main Window --
MyWindow =  class(Turbine.UI.Lotro.Window);

	function MyWindow:Constructor()
	
		-- Base --
		Turbine.UI.Lotro.GoldWindow.Constructor(self);

		-- Set initial attributes --
		self:SetResizable(true);
		self:SetText("Exerceo");
		self:SetWantsUpdates(true);
		
		-- Show window --
		self:SetVisible(true);

		-- General --
		self.m_pad = 40;
		
		-- Label --
		self.m_text = Turbine.UI.Label();
		self.m_text:SetParent(self);
		self.m_text:SetPosition(self.m_pad / 2, self.m_pad);
		self.m_text:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
		
		-- Timer --
		self.m_previousTime = Turbine.Engine.GetLocalTime();
		self.m_actualTime = 0;
		self.m_run = false;
		
		-- Buttons --
		self.m_buttons = { };
		self.m_buttonsWidth = 60;
		
		for i = 1, 3, 1 do
			self.m_buttons[i] = Turbine.UI.Lotro.Button();
			self.m_buttons[i]:SetParent(self);
			self.m_buttons[i]:SetSize(self.m_buttonsWidth, self.m_pad);
		end
	
		self.m_buttons[1]:SetText("Start");
		self.m_buttons[2]:SetText("Stop");
		self.m_buttons[3]:SetText("Reset");
		
		-- Player --
		self.m_player = MyPlayer();
		self.m_text:SetText(self.m_player:GetResults());
		
		-- Chat --
		self.m_chat = Turbine.Chat;
		
		self.m_chat.Received = function(sender, args)
			
			if self.m_run == true then
			
				if self.m_player:IsInCombat() == true then
			
					local name = string.match(args.Message, "(%a+) ");
					
					local damage = self:SearchForValues(args.Message, "for");
				
					if damage > 0 then -- Someone hurt someone...
				
						taken = true;
						
						if name == self.m_player:GetName() then taken = false; end -- I did..

						self.m_player:UpdateDamage(damage, taken);
					
					else
					
						if name == self.m_player:GetName() then
					
							local heal = self:SearchForValues(args.Message, "restoring");
						
							if heal > 0 then
						
								self.m_player:UpdateHealing(heal);
						
							end
						
						end
						
					end
					
					self.m_text:SetText(self.m_player:GetResults());
				
				end
				
			end
			
		end
		
		-- Start --
		self.m_buttons[1].MouseClick = function(sender, args)
			
			self.m_run = true;
			self.m_player.m_state = "On"
			self.m_text:SetText(self.m_player:GetResults());
			
		end
		
		-- Stop --
		self.m_buttons[2].MouseClick = function(sender, args)
		
			self.m_run = false;
			self.m_player.m_state = "Off"
			self.m_text:SetText(self.m_player:GetResults());
			
		end
		
		-- Reset --
		self.m_buttons[3].MouseClick = function(sender, args)
		
			self.m_player:Reset();
			self.m_text:SetText(self.m_player:GetResults());
			
		end
		
	end
	
	function MyWindow:SearchForValues(message, previousWord)
	
		local thousands, hundreds = string.match(message, "%s*" .. previousWord .. "%s*(%d+),(%d+) ");
		
		local outValue = 0;
				
		if thousands == nil then
				
			hundreds = string.match(message, "%s*" .. previousWord .. "%s*(%d+) ");
			
			if hundreds ~= nil then
			
				outValue = tonumber(hundreds)
				
			else return 0; end
		
		else
				
			outValue = tonumber(thousands .. hundreds)
				
		end
	
		return outValue;
		
	end
	
	function MyWindow:SizeChanged(sender, args)
	
		-- Get new dimensions --
		local width, height = self:GetSize();
			
		-- Label --
		self.m_text:SetSize(width - (self.m_pad),  height - (self.m_pad * 2)); 
			
		-- Buttons --
		local buttonInitX = width / 2 - (self.m_buttonsWidth * 3) / 2;
		
		for i = 1, 3, 1 do
		
			self.m_buttons[i]:SetPosition(buttonInitX + (i - 1) * self.m_buttonsWidth, height - self.m_pad + 10);
			
		end
		
	end
	
	function MyWindow:Update(sender, args)

		self.m_actualTime = Turbine.Engine:GetLocalTime();
		
		if self.m_run == true then
		
			if self.m_actualTime ~= self.m_previousTime then
				
				self.m_previousTime = self.m_actualTime;
				
				--Turbine.Shell.WriteLine(tostring(self.m_player.m_inCombatSeconds));
				
				if self.m_player:IsInCombat() == true then
				
					self.m_player.m_inCombatSeconds = self.m_player.m_inCombatSeconds + 1;
					self.m_player:UpdateDPS();
					self.m_text:SetText(self.m_player:GetResults());
					
				end
				
			end
			
		end
		
	end

	
StatExWindow = MyWindow();
StatExWindow:SetSize(230, 240);